local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("soundNotifications", "AceEvent-3.0")
local LSN = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-SoundNotifications")

--------------------------------------
--- Copy global functions to local ---
--------------------------------------
local PlaySoundFile = PlaySoundFile

local defaults = {
	profile = {
		playerObjectiveComplete = true,
		playerQuestComplete = true,
		playerQuestFailed = true,
	},
}

local prdb
function module:OnInitialize()
	self.db = core.db:RegisterNamespace("soundNotifications", defaults, "Default")
	prdb = self.db.profile
end

module.modName = LSN["Sound Notifications"]
local options
function module:getOptions()
	if not options then
		options = {
			type = "group",
			name = module.modName,
			desc = module.modName,
			get = function(info) 
				return prdb[info[#info]]
			end,
			set = function(info, value)
				prdb[info[#info]] = value
			end,
			args = {
				playerObjectiveComplete = {
					name = LSN["Toggle player quests objectives complete notification"],
					type = "toggle",
					width = "full",
					order = 10,
				},
				playerQuestComplete = {
					name = LSN["Toggle player quests complete notification"],
					type = "toggle",
					width = "full",
					order = 11,
				},
				playerQuestFailed = {
					name = LSN["Toggle player quests failed notification"],
					type = "toggle",
					width = "full",
					order = 12,
				},
			},
		}
	end
	return options
end

function module:OnEnable()
	core.RegisterCallback(self, "playerQuestObjectiveProgress")
	core.RegisterCallback(self, "playerQuestComplete")
	core.RegisterCallback(self, "playerQuestFailed")
end

function module:OnDisable()
	core.UnregisterAllCallbacks(self)
end

function module:playerQuestObjectiveProgress(event, firstRun, commHandler, uid, objective, got, nGot)
	if nGot == core.questsObjectives[uid][objective].nNeed and prdb.playerObjectiveComplete and not firstRun and core.quests[uid].party[core.playerName].complete == 0 then
		PlaySoundFile([[Sound\Creature\Peon\PeonReady1.wav]])
	end
end


function module:playerQuestComplete(event, firstRun, commHandler, uid)
	if prdb.playerQuestComplete and not firstRun then
		PlaySoundFile([[Interface\Addons\tomQuest2\media\Peasant_work_done.mp3]])
	end
end

function module:playerQuestFailed(event, firstRun, commHandler, uid)
	if prdb.playerQuestFailed and not firstRun then
		PlaySoundFile([[Sound\Interface\igPlayerBind.wav]])
	end
end