local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("syncParty", "AceEvent-3.0", "AceTimer-3.0", "AceComm-3.0", "AceSerializer-3.0")
local LSP = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-SyncParty")

--------------------------------------
--- Copy global functions to local ---
--------------------------------------
local pairs, type = pairs, type
local format = string.format
local GetTime = GetTime
local UnitName = UnitName
local GetNumPartyMembers =  GetNumPartyMembers

local addonPrefix = "tomQ2"
local version = 48
local firstCompatibleVersion = 48
local lastIncompatibleVersion = 47
local syncParty = false

module.modName = LSP["Party Synchronization"]

function module:OnEnable()
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterComm(addonPrefix, "commHandler")
 	core.RegisterCallback(self, "addPlayerQuest", "sendMessage")
	core.RegisterCallback(self, "updateObjectiveDesc", "sendMessage")
 	core.RegisterCallback(self, "addPlayerQuestObjective", "sendMessage")
	core.RegisterCallback(self, "endPlayerQuest", "sendMessage")
	core.RegisterCallback(self, "playerQuestObjectiveProgress", "sendMessage")
	core.RegisterCallback(self, "playerQuestComplete", "sendMessage")
	core.RegisterCallback(self, "playerQuestFailed", "sendMessage")
	core.RegisterCallback(self, "playerQuestLost", "sendMessage")
	core.RegisterCallback(self, "endSync", "sendMessage")
	core.RegisterCallback(self, "initParty", "PARTY_MEMBERS_CHANGED")
end

function module:OnDisable()
	core.UnregisterAllCallbacks(self)
end

function module:PARTY_MEMBERS_CHANGED()
	core.numPartyMembers = GetNumPartyMembers() or 0
	local timeStamp = GetTime()
	local partySync = 0
	if core.numPartyMembers > 0 then
		for i=1, core.numPartyMembers do
			local partyName = UnitName('party'..i)
			if not core.party[partyName] then
				core.party[partyName] = {}
				core.party[partyName].status = 0
			end
			if not core.party[partyName].timeStamp then
				module:SendCommMessage(addonPrefix, module:Serialize("v", version, firstCompatibleVersion, core.playerClass), "WHISPER", partyName)
			end
			core.party[partyName].timeStamp = timeStamp
			partySync = partySync + core.party[partyName].status
		end
	end

	for partyName in pairs(core.party) do
		if core.party[partyName].timeStamp ~= timeStamp then
			core.cbh:Fire("initTrackedQuest")
			for uid in pairs(core.quests) do
				if core.quests[uid].party[partyName] then
					core:questLost(partyName, false, uid)
				end
			end
			core.party[partyName] = nil
		end
	end
	if partySync == 0 then
		syncParty = false
	else
		syncParty = true
	end
end
-------------
--- INPUT ---
-------------
local comm = {
	v = function(sender, senderVersion, senderFirstCompatibleVersion, senderClass)
		if senderVersion <= lastIncompatibleVersion then
			DEFAULT_CHAT_FRAME:AddMessage(format(LSP["The tomQuest2 release (%d) used by %s is too old to synchronise with your release (%d) (min %d)"], senderVersion, sender, version, firstCompatibleVersion))
			return
		elseif version < senderFirstCompatibleVersion then
			DEFAULT_CHAT_FRAME:AddMessage(format(LSP["Your tomQuest2 release (%d) is too old to synchorise with release (%d) of %s (min %d)"], version, senderVersion, sender, firstCompatibleVersion))
			return
		elseif version < senderVersion then
			DEFAULT_CHAT_FRAME:AddMessage(format(LSP["New release available (%d) ask %s where to find it"], senderVersion, sender))
		end
		core.party[sender] = core.party[sender] or {}
		core.party[sender].status = 1
		core.party[sender].class = senderClass
		core.party[sender].color = core.classColors[senderClass]
		syncParty = true
		module:SendCommMessage(addonPrefix, module:Serialize("w", version, firstCompatibleVersion, core.playerClass), "WHISPER", sender)
		for uid in pairs(core.quests) do
			if core.quests[uid].party[core.playerName] then
				module:SendCommMessage(addonPrefix, module:Serialize("q", uid, core.quests[uid].title, core.quests[uid].shortTag, core.quests[uid].level, core.quests[uid].numObjectives, core.quests[uid].party[core.playerName].complete or 0), "WHISPER", sender, "BULK")
				local begun = 0
				if core.questsObjectives[uid] then
					for objective in pairs(core.questsObjectives[uid]) do
						if core.questsObjectives[uid][objective].party[core.playerName] then
							module:SendCommMessage(addonPrefix, module:Serialize("o", uid, objective, core.questsObjectives[uid][objective].desc, core.questsObjectives[uid][objective].qType, core.questsObjectives[uid][objective].party[core.playerName].got, core.questsObjectives[uid][objective].need, core.questsObjectives[uid][objective].party[core.playerName].nGot, core.questsObjectives[uid][objective].nNeed), "WHISPER", sender, "BULK")
							if core.questsObjectives[uid][objective].party[core.playerName].nGot ~= 0 then
								begun = 1
							end
						end
					end
				end
				module:SendCommMessage(addonPrefix, module:Serialize("r", uid, begun), "WHISPER", sender, "BULK")
			end
		end
	end,
	w = function(sender, senderVersion, senderFirstCompatibleVersion, senderClass)
		if senderVersion <= lastIncompatibleVersion then
			DEFAULT_CHAT_FRAME:AddMessage(format(LSP["The tomQuest2 release (%d) used by %s is too old to synchronise with your release (%d) (min %d)"], senderVersion, sender, version, firstCompatibleVersion))
			return
		elseif version < senderFirstCompatibleVersion then
			DEFAULT_CHAT_FRAME:AddMessage(format(LSP["Your tomQuest2 release (%d) is too old to synchorise with release (%d) of %s (min %d)"], version, senderVersion, sender, firstCompatibleVersion))
			return
		elseif version < senderVersion then
			DEFAULT_CHAT_FRAME:AddMessage(format(LSP["New release available (%d) ask %s where to find it"], senderVersion, sender))
		end
		core.party[sender] = core.party[sender] or {}
		core.party[sender].status = 1
		core.party[sender].class = senderClass
		core.party[sender].color = core.classColors[senderClass]
		syncParty = true
	end,
	q = function(sender, uid, title, shortTag, level, numObjectives, complete)
		if not core.party[sender] or core.party[sender].status == 0 then return end
		if not core.quests[uid] and not core.questsZones[LSP["PARTYQUESTSZONE"]] then
			core:addZone(LSP["PARTYQUESTSZONE"])
		end
		core:addQuest(sender, false, uid, title, shortTag, level, numObjectives, complete, LSP["PARTYQUESTSZONE"], nil)
		if not core.quests[uid].objective or not core.quests[uid].description then
			module:ScheduleTimer("checkQuestObjective", 60, module:Serialize(uid, sender))
		end
	end,
	o = function(sender, uid, objective, desc, qType, got, need, nGot, nNeed)
		if not core.party[sender] or not core.quests[uid] or not core.quests[uid].party[sender] or not core.questsObjectives[uid] or core.party[sender].status == 0 then return end
		core:addQuestObjective(sender, false, uid, objective, desc, qType, got, need, nGot, nNeed)
	end,
	r = function(sender, uid, numObjectives, started)
		if not core.party[sender] or not core.quests[uid] or not core.quests[uid].party[sender] or core.party[sender].status == 0 then return end
		core.cbh:Fire("updateQuestsTooltip")
		core.cbh:Fire("endPartyQuest", uid, started)
	end,
	m = function(sender, uid)
		if not core.party[sender] or not core.quests[uid] or not core.quests[uid].party[sender] or core.quests[uid].objective or core.party[sender].status == 0 then return end
		module:SendCommMessage(addonPrefix, module:Serialize("n", uid, core.quests[uid].objective), "WHISPER", sender, "BULK")
	end,
	n = function(sender, uid, objective)
		if not core.party[sender] or not core.quests[uid] or core.quests[uid].objective or core.party[sender].status == 0 then return end
		core.quests[uid].objective = objective
		core.cbh:Fire("questObjectiveAdded", uid)
	end,
	s = function(sender, uid)
		if not core.party[sender] or not core.quests[uid] or not core.quests[uid].party[sender] or core.quests[uid].description or core.party[sender].status == 0 then return end
		module:SendCommMessage(addonPrefix, module:Serialize("t", uid, core.quests[uid].description), "WHISPER", sender, "BULK")
	end,
	t = function(sender, uid, description)
		if not core.party[sender] or not core.quests[uid] or core.quests[uid].description or core.party[sender].status == 0 then return end
		core.quests[uid].description = description
		core.cbh:Fire("questDescriptionAdded", uid)
	end,
	p = function(sender, uid, objective, got, nGot)
		if not core.party[sender] or not core.quests[uid] or core.party[sender].status == 0 then return end
		core:questObjectiveProgress(sender, false, uid, objective, got, nGot)
	end,
	c = function(sender, uid)
		if not core.party[sender] or not core.quests[uid] or not core.quests[uid].party[sender] or core.party[sender].status == 0 then return end
		core:questComplete(sender, false, uid)
	end,
	f = function(sender, uid)
		if not core.party[sender] or not core.quests[uid] or not core.quests[uid].party[sender] or core.party[sender].status == 0 then return end
		core:questFailed(sender, false, uid)
	end,
	l = function(sender, uid)
		if not core.party[sender] or not core.quests[uid] or not core.quests[uid].party[sender] or core.party[sender].status == 0 then return end
		core:questLost(sender, false, uid)
	end,
}

function module:commHandler(prefix, message, distribution, sender)
	if sender ~= core.playerName then
		module:onCommand(sender, module:Deserialize(message))
	end
end

function module:onCommand(sender, success, commHandler, ...)
	if not success then
		return
	elseif comm[commHandler] then
		comm[commHandler](sender, ...)
	end
end

function module:checkQuestObjective(arg)
	local success, uid, sender = module:Deserialize(arg)
	if not success or not uid or not core.quests[uid] or not sender or not core.party[sender] then return end
	if not core.quests[uid].objective then
		module:SendCommMessage(addonPrefix, module:Serialize("m", uid), "WHISPER", sender, "BULK")
	end
	if not core.quests[uid].description then
		module:SendCommMessage(addonPrefix, module:Serialize("s", uid), "WHISPER", sender, "BULK")
	end
end
--------------
--- OUTPUT ---
--------------
function module:sendMessage(event, firstRun, ...)
	if core.numPartyMembers > 0 and syncParty and not firstRun then
		module:SendCommMessage(addonPrefix, module:Serialize(...), "PARTY", nil, "BULK")
	end
end